"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const pm2_1 = __importDefault(require("pm2"));
const path_1 = __importDefault(require("path"));
const config_1 = __importDefault(require("../config/config"));
const child_process_1 = require("child_process");
const chai_1 = require("chai");
const bluebird_1 = require("bluebird");
const CLIStart = `node ${path_1.default.join(__dirname, '../../bin/CLI')}`;
const pm2Promise = bluebird_1.Promise.promisifyAll(pm2_1.default);
function runCliCommand(command, target = __dirname) {
    return new bluebird_1.Promise((resolve, reject) => {
        const errorLog = [];
        const stdoutLog = [];
        const runCommand = child_process_1.exec(command, { cwd: target });
        runCommand.stderr.on('data', errorData => errorLog.push(errorData));
        runCommand.stdout.on('data', data => stdoutLog.push(data));
        runCommand.on('error', error => reject(error));
        runCommand.on('exit', code => {
            if (code !== 0) {
                reject(errorLog.join('\n'));
            }
            else {
                resolve({ status: 'success', stdout: stdoutLog.join('\n') });
            }
        });
    });
}
describe('commands module test', () => {
    const testDappsPath = path_1.default.join(__dirname, './testData/testDapps');
    const targetProjectPath = `./testData/testDapps/testDapp_${Math.floor(Math.random() * (1000 - 100)) + 100}`;
    before(() => {
        if (!fs_1.default.existsSync(testDappsPath)) {
            fs_1.default.mkdirSync(testDappsPath);
        }
    });
    after(async () => { await pm2Promise.disconnectAsync(); });
    it('version', async () => {
        // Arrange
        const versionOut = `CLI version: ${require(config_1.default.packageJSON).version}\n`;
        const versionCommand = `${CLIStart} -V`;
        // Act
        const viewTemplates = await runCliCommand(versionCommand);
        if (viewTemplates.status === 'success') {
            // Assert
            chai_1.expect(viewTemplates.stdout).to.be.eq(versionOut);
        }
    });
    it('list templates', async () => {
        // Arrange
        const checkTextTemplate = config_1.default.templates[0].name;
        const startCommand = `${CLIStart} list`;
        // Act
        const viewTemplates = await runCliCommand(startCommand);
        // Assert
        chai_1.expect(viewTemplates).to.have.property('status');
        chai_1.expect(viewTemplates.status).to.be.eq('success');
        chai_1.expect(viewTemplates.stdout).to.include(checkTextTemplate);
    });
    it('create new project', async () => {
        // Arrange
        const targetProjectTemplate = 'daocasino/dc-sdk-example';
        const startCommand = `${CLIStart} create ${targetProjectTemplate} ${targetProjectPath} -y`;
        // Act
        const createProject = await runCliCommand(startCommand);
        if (createProject.status === 'success') {
            const checkExistProject = fs_1.default.existsSync(path_1.default.join(__dirname, targetProjectPath, 'node_modules'));
            // Assert
            chai_1.expect(checkExistProject).to.be.eq(true);
        }
    });
    it('start enviroment', async () => {
        // Arrange
        const startCommand = `${CLIStart} start`;
        // Act
        const startEnviroment = await runCliCommand(startCommand, path_1.default.join(__dirname, targetProjectPath));
        if (startEnviroment.status === 'success') {
            const processList = await pm2Promise.listAsync();
            // Assert
            chai_1.expect(processList.length).to.be.eq(2);
        }
    });
    it('stop enviroment', async () => {
        // Arrange
        const stopCommand = `${CLIStart} stop`;
        // Act
        const stopEnviroment = await runCliCommand(stopCommand, path_1.default.join(__dirname, targetProjectPath));
        if (stopEnviroment.status === 'success') {
            const processList = await pm2Promise.listAsync();
            // Assert
            chai_1.expect(processList.length).to.be.eq(0);
        }
    });
});
//# sourceMappingURL=main.test.js.map